/*
 * control.c
 * Source file for controller functions
 * (sell, refund, inventory check, daily report)
 */

#include "daily.h"
#include "display.h"
#include "input.h"
#include "inven.h"
#include "refund.h"
#include "sell.h"

/*
 * sell()
 * 메인 컨트롤러의 한 부분입니다. 컨트롤러 중 "판매" 상황을 나타냅니다.
 * 리턴값: 판매 성공 시 1, 실패 시 0을 리턴하도록 만들어 주세요.
 */
int sell()
{
    INVEN_LIST *current_inventory;
    INPUT cashier_input;
    PRODUCT *selected_product = NULL;
    char *barcode_input;
    SELL_LIST *list = sell_init();

    current_inventory = synchronize();

    while (1)
    {
        system("clear");
        display_sell_customer_screen(list);
        display_sell_cashier_screen(list);
        display_sell_cashier_menu();

        cashier_input = get_cashier_input();

        if (cashier_input == QUANTITY_PLUS)
        {
            if (selected_product == NULL)
            {
                fputs("선택한 상품이 없습니다. \n", stderr);
                sleep(1);
                continue;
            }

            if (!get_product_amount(current_inventory, selected_product->name))
            {
                display_out_of_stock_message(selected_product->name);
            }
            else
            {
                product_quantity_plus(list, find_node(list, selected_product->name),
                                      find_inventory_node(current_inventory, selected_product->name));
            }
        }
        else if (cashier_input == QUANTITY_MINUS)
        {
            if (selected_product == NULL)
            {
                fputs("선택한 상품이 없습니다. \n", stderr);
                sleep(1);
                continue;
            }
            product_quantity_minus(list, find_node(list, selected_product->name),
                                   find_inventory_node(current_inventory, selected_product->name));
        }
        else if (cashier_input == BARCODE_SELL)
        {
            barcode_input = get_barcode_procduct_input();
            selected_product = get_product_barcode(barcode_input);
            if (selected_product == NULL)
            {
                fputs("바코드 번호가 유효하지 않습니다. \n", stderr);
                sleep(1);
                continue;
            }

            if (find_node(list, selected_product->name) != NULL)
            {
                product_quantity_plus(list, find_node(list, selected_product->name),
                                      find_inventory_node(current_inventory, selected_product->name));
            }
            else
            {
                add_sell_node(list, selected_product);
                product_quantity_plus(list, find_node(list, selected_product->name),
                                      find_inventory_node(current_inventory, selected_product->name));
            }
        }
        else if (cashier_input == RECEIPT_SELL)
        {
            perform_purchase(list);
            print_sell_receipt(list);
            add_sale_management(list);
            update_stock(current_inventory);
            return 1;
        }
        else
        {
            fputs("ERROR: invalid input. \n", stderr);
            return 0;
        }
    }
    return 0;
}
/*
 * refund()
 * 메인 컨트롤러의 한 부분입니다. 컨트롤러 중 "환불" 상황을 나타냅니다.
 * 리턴값: 환불 성공 시 1, 실패 시 0을 리턴하도록 만들어 주세요.
 */
int refund()
{
    PRODUCT current_product;
    INVEN_LIST *current_inventory;
    SELL_LIST *to_refund;
    char sale_receipt_name[50];
    char product_name_read[20];
    int product_price_read;
    int product_quantity_read;
    int price_read;
    char buffer[100];
    FILE *fp;
    int i;

    current_inventory = synchronize();
    to_refund = sell_init();

    system("clear");
    display_refund_cashier_menu();

    strcpy(sale_receipt_name, get_barcode_receipt_input());

    fp = fopen(sale_receipt_name, "r");
    if (fp == NULL)
    {
        display_error_message();
        return 0;
    }

    /* read 3 times and discard */
    for (i = 0; i < 3; i++)
    {
        fgets(buffer, sizeof(buffer), fp);
        memset(buffer, 0, sizeof(buffer));
    }

    /* TODO make refund list */
    while (!feof(fp))
    {
        fscanf(fp, "%s %d %d %d", product_name_read, &product_price_read, &product_quantity_read, &price_read);
        if (strcmp(product_name_read, "총") == 0)
        {
            break;
        }

        strcpy(current_product.name, product_name_read);
        current_product.price = product_price_read;

        add_sell_node(to_refund, &current_product);
        find_node(to_refund, current_product.name)->product_quantity = product_quantity_read;

        to_refund->total_price += price_read;
    }
    fclose(fp);

    system("clear");
    display_refund_customer_screen(to_refund);
    display_refund_cashier_screen(to_refund);

    /* refund */
    display_refund_cashier_menu();
    printf("환불을 진행하시겠습니까? (1: 예, 2: 아니오) : ");
    if (get_cashier_input())
    {
        SELL_NODE *curr;
        char filename[30];

        /* restore server */
        curr = to_refund->head->next;
        while(curr != to_refund->tail)
        {
            find_inventory_node(current_inventory, curr->product_name)->product->stock_amount += curr->product_quantity;
            curr = curr->next;
        }

        strcpy(filename, "refund_");
        strcat(filename, get_current_datetime());
        strcat(filename, ".txt");

        fp = fopen(filename, "w");
        if (fp == NULL)
        {
            fputs("ERROR: Cannot open file \n", stderr);
            return 0;
        }

        fprintf(fp, "%s %s", "영수증 번호 : ", get_current_datetime());
        fputs("\n", fp);
        fprintf(fp, "%s %s", "날짜 : ", get_current_date());
        fputs("\n", fp);
        fputs("환불 상품 \n", fp);

        curr = to_refund->head->next;
        while(curr != to_refund->tail)
        {
            fprintf(fp, "%s %d %d %d", curr->product_name, curr->product_price, curr->product_quantity, curr->product_price * curr->product_quantity);
            fputs("\n", fp);
            curr = curr->next;
        }
        fprintf(fp, "%s %d", "총 환불액 : ", to_refund->total_price);
        fputs("\n", fp);

        fclose(fp);
    }
    else
    {
        return 0;
    }
    /* add refund mangement */
    add_refund_management(to_refund);

    /* update server */
    update_stock(current_inventory);
    return 1;
}
/*
 * check_stock()
 * 메인 컨트롤러의 한 부분입니다. 컨트롤러 중 "재고 확인" 상황을 나타냅니다.
 * 리턴 값: 재고 확인 성공 시 1, 실패 시 0을 리턴하도록 만들어 주세요
 */
int check_stock()
{
    INVEN_LIST *inventory = synchronize();
    if (inventory == NULL)
    {
        return 0;
    }
    else
    {
        display_inventory(inventory);
        return print_stock(inventory);
    }
}
