/*
 * daily.c
 * Source file containing functions about daily report
 */

#include "daily.h"

/*
 * time_function()
 * Thread start routine. Separate function dealing with time
 * Makes time increment, calls for daily report when day renews
 * 
 * Parameter(s): void
 * Return Type: void *
 */
void *time_function(void *data)
{
    int *status;
    int flag = 0;

    while (1)
    {
        status = (int *)data;
        if (!(*status))
        {
            return status;
        }

        unsigned int year = atoi(CURRENT_TIME.year);
        unsigned int month = atoi(CURRENT_TIME.month);
        unsigned int date = atoi(CURRENT_TIME.date);
        unsigned int hour = atoi(CURRENT_TIME.hour);
        unsigned int minute = atoi(CURRENT_TIME.minute);

        /* 1 second = 8 minute */
        minute += 8;
        if (minute > 59)
        {
            hour++;
            minute %= 60;
        }
        if (hour > 23)
        {
            *status = 2;
            printf("*** 하루가 지났습니다 ***");
            printf("메인 화면으로 나가시면 정산이 진행됩니다. \n");
            flag = 1;
            date++;
            hour = 0;
        }
        if (date > 30)
        {
            month++;
            date = 1;
        }
        if (month > 12)
        {
            year++;
            month = 1;
        }

        // integer back to string
        sprintf(CURRENT_TIME.year, "%04d", year);
        sprintf(CURRENT_TIME.month, "%02d", month);
        sprintf(CURRENT_TIME.date, "%02d", date);
        sprintf(CURRENT_TIME.hour, "%02d", hour);
        sprintf(CURRENT_TIME.minute, "%02d", minute);

        if(flag)
        {
            renew_server();
        }

        sleep(1);
    }
}

char *get_current_date()
{
    char *date = (char *)malloc(9);
    strcpy(date, CURRENT_TIME.year);
    strcat(date, CURRENT_TIME.month);
    strcat(date, CURRENT_TIME.date);
    return date;
}

char *get_current_datetime()
{
    char *datetime = (char *)malloc(13);
    strcpy(datetime, CURRENT_TIME.year);
    strcat(datetime, CURRENT_TIME.month);
    strcat(datetime, CURRENT_TIME.date);
    strcat(datetime, CURRENT_TIME.hour);
    strcat(datetime, CURRENT_TIME.minute);
    return datetime;
}
/*
 * print_daily_report()
 * Prints daily report when time is 00:00
 * If sale or refund is in progress, print_daily_report() waits until previous process ends
 * 
 * Parameter(s): 
 * Return Type: void
 */
void print_daily_report()
{
    char *command = (char *)malloc(100);
    char datetime[32] = {0};
    char file_name[20] = "settle_";

    /* making date */
    strncat(datetime, CURRENT_TIME.year, 4);
    strncat(datetime, CURRENT_TIME.month, 2);
    strncat(datetime, CURRENT_TIME.date, 2);

    /* making filename */
    strncat(file_name, datetime, 8);
    strncat(file_name, ".txt", 4);
    strncat(datetime, "_sale_management.txt", sizeof(datetime) - 8);

    /* generating command */
    strncpy(command, "cp ", 3);
    strcat(command, datetime);
    strcat(command, " ");
    strcat(command, file_name);

    printf("%s \n", command);

    /* executing command */
    system(command);
}
/*
 * renew_server()
 * Renews server when time is 00:00
 * 
 * Parameter: void
 * Return Type: void
 */
int renew_server()
{
    FILE *fp;
    char file_name[50] = {0};
    char *current_date = get_current_date();

    strcpy(file_name, current_date);
    strcat(file_name, "_product.txt");
    fp = fopen(file_name, "w");
    if (fp == NULL)
    {
        fputs("ERROR: Failed to open file. \n", stderr);
        return 0;
    }
    fputs("과자 1000 001 100\n", fp);
    fputs("아이스크림 1500 010 100\n", fp);
    fputs("과일 3000 011 100\n", fp);
    fputs("물 500 100 100\n", fp);
    fputs("라면 800 101 100\n", fp);
    fputs("음료수 1200 110 100\n", fp);
    fputs("커피 2000 111 100\n", fp);

    fclose(fp);

    memset((void *)file_name + 8, 0, 20);

    /* YYYYMMDD_sale_management.txt */
    strcat(file_name, "_sale_management.txt");
    fp = fopen(file_name, "w");
    if (fp == NULL)
    {
        fputs("ERROR: Failed to open file. \n", stderr);
        return 0;
    }

    free(current_date);
    fclose(fp);
    return 1;
}