/*
 * display.c
 * File containing function about displaying
 */

#include "display.h"
#include "sell.h"

/*
 * display_main_menu()
 * Main cashier screen display
 * 
 * Parameter(s): void
 * Return type: void
 */
void display_main_menu()
{
    printf("******************************** \n");
    printf("*    POINT OF SALE TERMINAL    * \n");
    printf("******************************** \n");
    printf("*  1. Turn Off machine         * \n");
    printf("*  2. SELL                     * \n");
    printf("*  3. REFUND                   * \n");
    printf("*  4. CHECK STOCK              * \n");
    printf("******************************** \n");
}
/*
 * display_sell_cashier_menu()
 * Sell state menu display - cashier screen
 * 
 * Parameter(s): void
 * Return type: void
 */
void display_sell_cashier_menu()
{
    // TODO display of product list, total price
    printf("******************************** \n");
    printf("*          SELL STATE          * \n");
    printf("******************************** \n");
    printf("*  1. Increase Quantity        * \n");
    printf("*  2. Decrease Quantity        * \n");
    printf("*  3. BARCODE                  * \n");
    printf("*  4. RECEIPT                  * \n");
    printf("******************************** \n");
}
void display_sell_cashier_screen(SELL_LIST *list)
{
    SELL_NODE *temp = list->head->next;
    printf("-------------------------------- \n");
    printf("|        CASHIER SCREEN        | \n");
    printf("-------------------------------- \n\n");

    printf(" 상품명       \t수량  \t단가  \t총 판매액 \n");
    printf("------------------------------------------ \n");
    while (temp != list->tail)
    {
        printf(" %11s\t%2d\t%5d\t%d \n", temp->product_name, temp->product_quantity, temp->product_price, temp->product_price * temp->product_quantity);
        temp = temp->next;
    }
    printf("\nTotal Price : %5d \n\n\n", list->total_price);
}
/*
 * display_sell_customer_menu()
 * Sell state menu display - customer screen
 * 
 * Parameter(s): void
 * Return type: void
 */
void display_sell_customer_screen(SELL_LIST *list)
{
    SELL_NODE *temp = list->head->next;
    printf("--------------------------------- \n");
    printf("|        CUSTOMER SCREEN        | \n");
    printf("--------------------------------- \n\n");

    printf(" 상품명       \t수량  \t단가  \t총 판매액 \n");
    printf("------------------------------------------ \n");
    while (temp != list->tail)
    {
        printf(" %11s\t%2d\t%5d\t%d \n", temp->product_name, temp->product_quantity, temp->product_price, temp->product_price * temp->product_quantity);
        temp = temp->next;
    }
    printf("\nTotal Price : %5d \n\n\n", list->total_price);
}
/*
 * display_refund_cashier_menu()
 * Refund state menu display - cashier screen
 * 
 * Parameter(s): void
 * Return type: void
 */
void display_refund_cashier_menu()
{
    printf("******************************** \n");
    printf("*         REFUND STATE         * \n");
    printf("******************************** \n");
}
void display_refund_cashier_screen(SELL_LIST *list)
{
    SELL_NODE *temp = list->head->next;
    printf("-------------------------------- \n");
    printf("|        CASHIER SCREEN        | \n");
    printf("-------------------------------- \n\n");

    printf(" 상품명       \t수량  \t단가  \t총 환불액 \n");
    printf("------------------------------------------ \n");
    while (temp != list->tail)
    {
        printf(" %11s\t%2d\t%5d\t%d \n", temp->product_name, temp->product_quantity, temp->product_price, temp->product_price * temp->product_quantity);
        temp = temp->next;
    }
    printf("\nTotal Price : %5d \n\n\n", list->total_price);
}
/*
 * display_refund_customer_menu()
 * Refund state menu display - customer screen
 * 
 * Parameter(s): void
 * Return type: void
 */
void display_refund_customer_screen(SELL_LIST *list)
{
    SELL_NODE *temp = list->head->next;
    printf("--------------------------------- \n");
    printf("|        CUSTOMER SCREEN        | \n");
    printf("--------------------------------- \n\n");
    printf(" 상품명       \t수량  \t단가  \t총 환불액 \n");
    printf("------------------------------------------ \n");
    while (temp != list->tail)
    {
        printf(" %11s\t%2d\t%5d\t%d \n", temp->product_name, temp->product_quantity, temp->product_price, temp->product_price * temp->product_quantity);
        temp = temp->next;
    }
    printf("\nTotal Price : %5d \n\n\n", list->total_price);
}
/*
 * display_error_message()
 * Displays error message when refund is impossible
 * 
 * Parameter(s): void
 * Return Type: void
 */
void display_error_message()
{
    printf("해당 영수증은 환불이 불가능 합니다 \n");
}
/*
 * display_out_of_stock_message
 * Displays error message when product is out of stock, during sell state
 * 
 * Parameter product_name: String - Name of product
 * Return Type: void
 */
void display_out_of_stock_message(char *product_name)
{
    printf("%s 의 개수가 부족합니다 \n", product_name);
}