#include "inven.h"
#include "daily.h"

#define FAIL 0
#define SUCCESS 1 // control.c 에서 성공 여부를 받기 위해서 return 값 true false로 만들어주려고

INVEN_LIST *synchronize()
{
    int i;
    INVEN_LIST *list;
    char filename[30];
    char buffer[100];
    char product_name_read[20];
    int product_price_read;
    int product_quantity_read;
    char product_barcode_read[4];

    FILE *fp;

    /* List creation */
    list = (INVEN_LIST *)malloc(sizeof(INVEN_LIST));
    list->head = (INVEN_NODE *)malloc(sizeof(INVEN_NODE));
    list->tail = (INVEN_NODE *)malloc(sizeof(INVEN_NODE));
    list->head->next = list->tail;
    list->tail->prev = list->head;

    list->head->product = NULL;
    list->tail->product = NULL;

    strcpy(filename, get_current_date());
    strcat(filename, "_product.txt");

    fp = fopen(filename, "r");
    if (fp == NULL)
    {
        fputs("ERROR: Cannot open file. \n", stderr);
        return NULL;
    }

    for (i = 0; i < 7; i++)
    {
        PRODUCT *new_product = (PRODUCT *)malloc(sizeof(PRODUCT));
        fscanf(fp, "%s %d %s %d", product_name_read, &product_price_read, product_barcode_read, &product_quantity_read);

        strcpy(new_product->name, product_name_read);
        new_product->price = product_price_read;
        new_product->stock_amount = product_quantity_read;
        strcpy(new_product->barcode_number, product_barcode_read);

        add_inventory_node(list, new_product);
    }
    return list;
}

void update_stock(INVEN_LIST *new_inven)
{
    char filename[30];
    INVEN_NODE *curr;
    FILE *fp;

    strcpy(filename, get_current_date());
    strcat(filename, "_product.txt");

    fp = fopen(filename, "w");
    if (fp == NULL)
    {
        fputs("ERROR: Cannot open file. \n", stderr);
        return;
    }

    curr = new_inven->head->next;
    while (curr != new_inven->tail)
    {
        fprintf(fp, "%s %d %s %d \n", curr->product->name, curr->product->price,
                curr->product->barcode_number, curr->product->stock_amount);
        curr = curr->next;
    }
    fclose(fp);
}

int add_inventory_node(INVEN_LIST *list, PRODUCT *product)
{
    INVEN_NODE *new_node = (INVEN_NODE *)malloc(sizeof(INVEN_NODE));
    new_node->next = list->tail;
    new_node->prev = list->tail->prev;
    new_node->prev->next = new_node;
    new_node->next->prev = new_node;

    /* add product info */
    new_node->product = product;

    return SUCCESS;
}

INVEN_NODE *find_inventory_node(INVEN_LIST *list, char *product_name)
{
    INVEN_NODE *temp = list->head->next;
    while (temp != list->tail)
    {
        if (strcmp(product_name, temp->product->name) == 0)
            return temp;

        temp = temp->next;
    }
    return NULL;
}

unsigned int get_product_amount(INVEN_LIST *list, char *product_name)
{
    INVEN_NODE *node = find_inventory_node(list, product_name);
    if (node == NULL)
    {
        fputs("ERROR: Product does not exist. \n", stderr);
        return 0;
    }
    return node->product->stock_amount;
}

int display_inventory(INVEN_LIST *list)
{
    INVEN_NODE *temp = list->head->next;
    while (temp != list->tail)
    {
        printf("상품명 : %s -> ", temp->product->name);
        printf("수량 : %d\n", temp->product->stock_amount);

        temp = temp->next; //다음 노드로 넘어감.
    }
    printf("\n"); //다음에 뭐올지 몰라서 그냥 개행.
    return SUCCESS;
}

int print_stock(INVEN_LIST *list)
{
    //stock_YYYYMMDD.txt 파일 생성. 파일명 & 수량
    FILE *fp;
    char filename[21];
    INVEN_NODE *temp = list->head->next;

    strcpy(filename, "stock_");
    strcat(filename, get_current_date());
    strcat(filename, ".txt");

    fp = fopen(filename, "w");
    if (fp == NULL)
    {
        printf("cannot print stock!\n");
        return FAIL;
    }

    while (temp != list->tail)
    {
        fprintf(fp, "상품명 : %s 가격 : %d 수량 : %d\n", temp->product->name, temp->product->price, temp->product->stock_amount);

        temp = temp->next; //다음 노드로 넘어감.
    }

    fclose(fp);

    return SUCCESS;
}