/*
 * sell.c
 * Source file for functions about sell procedure
 */

#include "sell.h"
#include "daily.h"
#include "inven.h"

SELL_LIST *sell_init()
{
    SELL_LIST *new_list = (SELL_LIST *)malloc(sizeof(SELL_LIST));

    new_list->head = (SELL_NODE *)malloc(sizeof(SELL_NODE));
    new_list->tail = (SELL_NODE *)malloc(sizeof(SELL_NODE));

    new_list->head->next = new_list->tail;
    new_list->tail->prev = new_list->head;

    new_list->head->prev = NULL;
    new_list->tail->next = NULL;

    new_list->total_price = 0;

    return new_list;
}

int is_sell_list_empty(SELL_LIST *list)
{
    if (list->head->next == list->tail)
        return 1;
    else
        return 0;
}
/*
 * add_sell_node()
 * Adds new node to sell list
 * 
 * Parameter: void
 * Return type: void
 */
int add_sell_node(SELL_LIST *list, PRODUCT *product)
{
    SELL_NODE *new_node = (SELL_NODE *)malloc(sizeof(SELL_NODE));
    new_node->next = list->tail;
    new_node->prev = list->tail->prev;
    new_node->next->prev = new_node;
    new_node->prev->next = new_node;

    strcpy(new_node->product_name, product->name);
    new_node->product_quantity = 0;
    new_node->product_price = product->price;

    return 1;
}

SELL_NODE *find_node(SELL_LIST *list, char *product_name)
{
    SELL_NODE *temp = list->head->next;
    while (temp != list->tail)
    {
        if (strcmp(product_name, temp->product_name) == 0)
        {
            return temp;
        }
        temp = temp->next;
    }
    return NULL;
}

PRODUCT *get_product_barcode(char *barcode_num)
{
    char file_name[30];
    char product_name[15];
    int product_price;
    char barcode_read[4];
    int product_quantity;

    PRODUCT *product = (PRODUCT *)malloc(sizeof(PRODUCT));

    FILE *fp;

    strcpy(file_name, get_current_date());
    strcat(file_name, "_product.txt");

    fp = fopen(file_name, "r");
    if (fp == NULL)
    {
        fputs("ERROR: Failed to open file. \n", stderr);
        return NULL;
    }

    while (!feof(fp))
    {
        if (fscanf(fp, "%s %d %s %d", product_name, &product_price, barcode_read, &product_quantity) != 4)
        {
            fputs("ERROR: Failed to read values. \n", stderr);
            return NULL;
        }
        if (strcmp(barcode_num, barcode_read) == 0)
        {
            fclose(fp);
            strcpy(product->barcode_number, barcode_read);
            strcpy(product->name, product_name);
            product->price = product_price;
            product->stock_amount = product_quantity;

            return product;
        }
    }
    fclose(fp);
    return NULL;
}

int product_quantity_minus(SELL_LIST *list, SELL_NODE *node, INVEN_NODE *inven)
{
    if (node == NULL)
        return 0;
    if (node->product_quantity == 0)
        return 0;
    else
    {
        node->product_quantity -= 1;
        list->total_price -= node->product_price;
        inven->product->stock_amount += 1;

        if (node->product_quantity == 0)
        {
            // TODO remove node
        }
        return 1;
    }
}

int product_quantity_plus(SELL_LIST *list, SELL_NODE *node, INVEN_NODE *inven)
{
    node->product_quantity += 1;
    list->total_price += node->product_price;
    inven->product->stock_amount -= 1;
    return 1;
}

void perform_purchase(SELL_LIST *list)
{
    int price_input;
    int additional_price_input;
    printf("지불할 금액을 입력하세요 : ");
    scanf("%d", &price_input);

    while (price_input < list->total_price)
    {
        printf("금액이 부족합니다. 추가로 더 지불하세요 : ");
        scanf("%d", &additional_price_input);
        price_input += additional_price_input;
    }

    system("clear");

    display_cashier_screen(list, price_input);
    display_customer_screen(list->total_price, price_input);
    sleep(1);
}

void display_cashier_screen(SELL_LIST *list, int price_input)
{
    SELL_NODE *temp = list->head->next;

    printf("-------------------------------- \n");
    printf("|        CASHIER SCREEN        | \n");
    printf("-------------------------------- \n\n");

    printf("총 판매액 : %d \n", list->total_price);
    printf("현금 : %d \n", price_input);
    printf("거스름돈 : %d \n", price_input - list->total_price);

    printf(" 상품명       \t수량  \t단가  \t총 판매액 \n");
    printf("------------------------------------------ \n");
    while (temp != list->tail)
    {
        printf(" %11s\t%2d\t%5d\t%d \n", temp->product_name, temp->product_quantity, temp->product_price, temp->product_price * temp->product_quantity);
        temp = temp->next;
    }
    printf("\n\n\n");
}

void display_customer_screen(int total_price, int price_input)
{
    printf("--------------------------------- \n");
    printf("|        CUSTOMER SCREEN        | \n");
    printf("--------------------------------- \n\n");

    printf("총 판매액 : %d \n", total_price);
    printf("현금 : %d \n", price_input);
    printf("거스름돈 : %d \n", price_input - total_price);
}

int print_sell_receipt(SELL_LIST *list)
{
    FILE *fp;
    SELL_NODE *curr;
    char filename[22] = "sale_";
    char *current_datetime = get_current_datetime();

    strcat(filename, current_datetime);
    strcat(filename, ".txt");

    fp = fopen(filename, "w");
    if (fp == NULL)
    {
        fputs("ERROR: Cannot open file. \n", stderr);
        return 0;
    }

    fprintf(fp, "%s %s", "영수증 번호 : ", current_datetime);
    fputs("\n", fp);
    fprintf(fp, "%s %s", "날짜 : ", get_current_date());
    fputs("\n", fp);
    fputs("판매 상품 \n", fp);

    curr = list->head->next;
    while (curr != list->tail)
    {
        fprintf(fp, "%s %d %d %d", curr->product_name, curr->product_price, curr->product_quantity, curr->product_price * curr->product_quantity);
        fputs("\n", fp);
        curr = curr->next;
    }
    fprintf(fp, "%s %d", "총 판매액 : ", list->total_price);
    fputs("\n", fp);

    fclose(fp);
    return 1;
}

void add_sale_management(SELL_LIST *list)
{
    SELL_NODE *curr;
    char filename[30];
    FILE *fp;

    strcpy(filename, get_current_date());
    strcat(filename, "_sale_management.txt");

    fp = fopen(filename, "a");
    if (ftell(fp) == 0)
    {
        fprintf(fp, "%s %s", "날짜 : ", get_current_date());
        fputs("\n\n", fp);
    }

    fputs("판매 상품 \n", fp);

    curr = list->head->next;
    while (curr != list->tail)
    {
        fprintf(fp, "%s %d %d %d\n", curr->product_name, curr->product_price, curr->product_quantity, curr->product_price * curr->product_quantity);
        curr = curr->next;
    }
    fprintf(fp, "%s %d", "판매 금액 : ", list->total_price);
    fputs("\n\n", fp);

    fclose(fp);
}